﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Reflection;
using System.ServiceModel;
using System.Text;
using System.Web;
using System.Xml;
using System.Linq;
using System.Xml.Linq;
using System.Xml.Xsl;
using RemoteTestRunner.WCFService;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace RemoteTestRunner.BusinessLogic
{
    public class RtrLib : IRTRService
    {
        #region Private Attributes

        private const string MultiAssertFileSuffix = ".MultiAssertOutput.xml";

        private DataAccess.DataAccess _dataAccess;

        #endregion

        #region IRTRService Properties

        public DataAccess.DataAccess DataAccess
        {
            get
            {
                if (_dataAccess == null)
                {
                    _dataAccess = new RemoteTestRunner.DataAccess.DataAccess(RtrSettings.DataPath);
                }

                return _dataAccess;
            }

            private set
            {
                _dataAccess = value;
            }
        }

        #endregion

        #region Static Methods

        /// <summary>
        /// Starts an e-mail message using the default e-mail client.
        /// </summary>
        /// <param name="subject">The subject.</param>
        /// <param name="messageBody">The message body.</param>
        public static void StartEmail(string subject, string messageBody)
        {
            Process.Start(
                string.Format("mailto:?subject={0}&body={1}", HttpUtility.UrlPathEncode(subject), messageBody));
        }

        /// <summary>
        /// Starts an e-mail message using the default e-mail client.
        /// </summary>
        /// <param name="testResult">An instance of a <see cref="TestResult"/> objected
        /// containing the details for the test run.</param>
        public static void StartEmail(TestResult testResult)
        {
            string subject = "Service Level Test Notification [{0}]";
            string message = "Test Run: {6}%0ATotal: {0}  Executed: {1}  Passed: {2}  Failed: {3}  Timed Out: {4}  Inconclusive: {5} %0A%0A";
            int dataRow;

            message = string.Format(
                message,
                testResult.TotalCount,
                testResult.TotalExecuted,
                testResult.TotalPassed,
                testResult.TotalFailed,
                testResult.TotalTimedOut,
                testResult.TotalInconclusive,
                testResult.TestRunName);

            foreach (TestResultInfo item in testResult.TestResults)
            {
                // Do not e-mail details on passed tests
                if (!item.Passed)
                {
                    message += item.ToString() + " %0A%0A";
                }

                if (item.DataDrivenResults != null)
                {
                    dataRow = 0;

                    foreach (TestResultInfo ddItem in item.DataDrivenResults)
                    {
                        dataRow++;

                        if (!ddItem.Passed)
                        {
                            message += string.Format("     ==> [row: {0}]", dataRow) +
                                ddItem.ToString() + " %0A%0A";
                        }
                    }
                }
            }

            if (testResult.Outcome == "Completed")
            {
                subject = string.Format(subject, "Passed");
            }
            else if (testResult.TotalCount > 0)
            {
                subject = string.Format(subject, "Failed");
            }
            else
            {
                subject = string.Format(subject, "No Tests Run");
            }

            RtrLib.StartEmail(subject, message);
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// Gets a list of test projects.
        /// </summary>
        /// <returns>A <see cref="DataTable"/> containing a list of all test projects.</returns>
        public DataTable GetTestProjects()
        {
            return DataAccess.GetProjects();
        }

        /// <summary>
        /// Deletes the specified test.
        /// </summary>
        /// <param name="testId">The ID of the test to be deleted.</param>
        public void DeleteTest(int testId)
        {
            DataAccess.DeleteTest(testId);
        }

        /// <summary>
        /// Gets a list of tests.
        /// </summary>
        /// <returns>A <see cref="DataTable"/> containing a list of all tests.</returns>
        public DataTable GetTests()
        {
            return DataAccess.GetTests();
        }

        /// <summary>
        /// Gets the specified test row.
        /// </summary>
        /// <param name="testId">The ID of the test to return.</param>
        /// <returns>A <see cref="DataRow"/> containing the specified test information.</returns>
        public TestInfo GetTestRow(int testId)
        {
            DataRow row = DataAccess.GetTestRow(testId);

            if (row != null)
            {
                return new TestInfo
                           {
                               Id = (int)row["ID"],
                               ProjectId = (int)row["ProjectID"],
                               Name = row["Name"].ToString(),
                               AssemblyPath = row["AssemblyPath"].ToString(),
                               ConfigPath = row["ConfigPath"].ToString(),
                               CreatedBy = row["CreatedBy"].ToString(),
                               CreatedDate = DateTime.Parse(row["CreatedDate"].ToString()),
                               ModifiedBy = row["ModifiedBy"].ToString(),
                               ModifiedDate = DateTime.Parse(row["ModifiedDate"].ToString())
                           };
            }
            else
            {
                return null;
            }
        }

        /// <summary>
        /// Gets a collection of data source information.
        /// </summary>
        /// <param name="appConfigPath">The application configuration file to be parsed.</param>
        /// <returns>A list of <see cref="DataSourceInfo"/> containing information for each
        /// data source found in the configuration file.</returns>
        public List<DataSourceInfo> GetDataSources(string appConfigPath)
        {
            List<DataSourceInfo> dataSources = new List<DataSourceInfo>();
            DataSourceInfo dsInfo;
            XmlDocument doc;
            XmlNode dsNameNode;
            XmlNode dsBaseUriNode;

            if (File.Exists(appConfigPath))
            {
                doc = new XmlDocument();
                doc.Load(appConfigPath);

                XmlNodeList dataSourceNodes = doc.SelectNodes("/configuration/microsoft.visualstudio.testtools/dataSources/add");

                foreach (XmlNode dsNode in dataSourceNodes)
                {
                    dsNameNode = doc.SelectSingleNode(
                        string.Format("/configuration/connectionStrings/add[@name='{0}']", dsNode.Attributes.GetNamedItem("connectionString").Value));

                    dsInfo = new DataSourceInfo()
                    {
                        Name = Utilities.GetAttributeValue(dsNode.Attributes, "name"),
                        DataSourcePath = Utilities.GetConnectionStringSetting(Utilities.GetAttributeValue(dsNameNode.Attributes, "connectionString"), ConnectionStringSetting.DataSource),
                        ConnectionStringName = Utilities.GetAttributeValue(dsNode.Attributes, "connectionString"),
                        DataTableName = Utilities.GetAttributeValue(dsNode.Attributes, "dataTableName"),
                        DataAccessMethod = Utilities.GetAttributeValue(dsNode.Attributes, "dataAccessMethod")
                    };

                    dsBaseUriNode = doc.SelectSingleNode(
                        string.Format(
                        "/configuration/appSettings/add[@key='{0}']",
                        "DDTCB:" + dsInfo.ConnectionStringName));

                    if (dsBaseUriNode != null)
                    {
                        dsInfo.DataSourceBaseUri = Utilities.GetAttributeValue(dsBaseUriNode.Attributes, "value");
                    }

                    dataSources.Add(dsInfo);
                }
            }

            return dataSources;
        }

        /// <summary>
        /// Gets the test data.
        /// </summary>
        /// <returns>
        /// A dataset containing the test configuration.
        /// </returns>
        public DataSet GetTestData()
        {
            return DataAccess.GetTestData();
        }

        /// <summary>
        /// Saves the test data.
        /// </summary>
        /// <param name="dataSet">The data set.</param>
        public void SaveTestData(DataSet dataSet)
        {
            DataAccess.SaveTestData(dataSet);
        }

        /// <summary>
        /// Parses the test results summary.
        /// </summary>
        /// <param name="testResultsPath">The test results path.</param>
        /// <returns>Test result summary.</returns>
        public TestResultSummary ParseTestResultsSummary(string testResultsPath)
        {
            TestResultSummary summary = new TestResultSummary();

            XmlNamespaceManager nsm;
            XmlDocument doc = GetTestResultXmlDocument(testResultsPath, out nsm);

            // Check the XML file to ensure it is an actual test results file
            if (doc.SelectSingleNode("ns:TestRun", nsm) == null)
            {
                return null;
            }

            // Store the path
            summary.Path = testResultsPath;

            summary.TestRunName = doc.SelectSingleNode("ns:TestRun", nsm).Attributes["name"].Value;
            summary.TestStartTime = DateTime.Parse(doc.SelectSingleNode("ns:TestRun/ns:Times", nsm).Attributes["start"].Value);
            summary.TestRunId = doc.SelectSingleNode("ns:TestRun", nsm).Attributes["id"].Value;

            return summary;
        }

        /// <summary>
        /// Parses the test results.
        /// </summary>
        /// <param name="testResultsPath">The test results path.</param>
        /// <returns></returns>
        public TestResult ParseTestResults(string testResultsPath)
        {
            TestResult testResult = new TestResult();
            TestResultInfo testResultInfo;
            TestResultInfo dataDrivenResultInfo;
            XmlNamespaceManager nsm;
            XmlDocument doc = GetTestResultXmlDocument(testResultsPath, out nsm);
            XmlNode unitTestNode;

            // Check the XML file to ensure it is an actual test results file
            if (doc.SelectSingleNode("ns:TestRun", nsm) == null)
            {
                return null;
            }

            // Store the path
            testResult.Path = testResultsPath;

            // Look for related XML files
            var multiAssertFiles =
                new List<string>(
                    System.IO.Directory.GetFiles(System.IO.Path.GetDirectoryName(testResult.Path), "*" + MultiAssertFileSuffix));
            multiAssertFiles.ForEach(file =>
                {
                    var multiAssertResult = new TestResultMultiAssertResult();
                    multiAssertResult.FileName = file;
                    multiAssertResult.FileContents = System.IO.File.ReadAllText(file);
                    testResult.MultiAssertResults.Add(multiAssertResult);
                });

            // Gather overall test statistics
            testResult.TestRunName = doc.SelectSingleNode("ns:TestRun", nsm).Attributes["name"].Value;
            testResult.TestStartTime = DateTime.Parse(doc.SelectSingleNode("ns:TestRun/ns:Times", nsm).Attributes["start"].Value);
            testResult.TestRunId = doc.SelectSingleNode("ns:TestRun", nsm).Attributes["id"].Value;
            testResult.Outcome = doc.SelectSingleNode("ns:TestRun/ns:ResultSummary", nsm).Attributes["outcome"].Value;

            testResult.TotalCount = int.Parse(doc.SelectSingleNode("ns:TestRun/ns:ResultSummary/ns:Counters", nsm).Attributes["total"].Value);
            testResult.TotalExecuted = int.Parse(doc.SelectSingleNode("ns:TestRun/ns:ResultSummary/ns:Counters", nsm).Attributes["executed"].Value);
            testResult.TotalPassed = int.Parse(doc.SelectSingleNode("ns:TestRun/ns:ResultSummary/ns:Counters", nsm).Attributes["passed"].Value);
            testResult.TotalFailed = int.Parse(doc.SelectSingleNode("ns:TestRun/ns:ResultSummary/ns:Counters", nsm).Attributes["failed"].Value);
            testResult.TotalTimedOut = int.Parse(doc.SelectSingleNode("ns:TestRun/ns:ResultSummary/ns:Counters", nsm).Attributes["timeout"].Value);
            testResult.TotalInconclusive = int.Parse(doc.SelectSingleNode("ns:TestRun/ns:ResultSummary/ns:Counters", nsm).Attributes["inconclusive"].Value);

            // Create a list of nodes that we'll be working with
            XmlNodeList unitTestResults = doc.SelectNodes("ns:TestRun/ns:Results/ns:UnitTestResult", nsm);

            // Iterate through each of the test results and gather the required information
            foreach (XmlNode unitTestResultNode in unitTestResults)
            {
                testResultInfo = new TestResultInfo();

                // Gather test-level properties
                testResultInfo.TestId = Utilities.GetAttributeValue(unitTestResultNode.Attributes, "testId");
                testResultInfo.ExecutionId = Utilities.GetAttributeValue(unitTestResultNode.Attributes, "executionId");
                testResultInfo.IsDataDriven = (Utilities.GetAttributeValue(unitTestResultNode.Attributes, "resultType") == "DataDrivenTest");
                testResultInfo.Outcome = Utilities.GetAttributeValue(unitTestResultNode.Attributes, "outcome");
                
                // default the message to the outcome.  If the test failed then the message will be updated with the failure message.
                testResultInfo.Message = testResultInfo.Outcome;

                // Gather method-level properties
                unitTestNode = doc.SelectSingleNode(string.Format("ns:TestRun/ns:TestDefinitions/ns:UnitTest[@id='{0}']/ns:TestMethod", testResultInfo.TestId), nsm);

                testResultInfo.MethodName = Utilities.GetAttributeValue(unitTestNode.Attributes, "name");
                testResultInfo.ClassName = Utilities.GetAttributeValue(unitTestNode.Attributes, "className");

                if (testResultInfo.IsDataDriven)
                { // The data-driven test results follow a slightly different format
                    XmlNodeList ddResults = unitTestResultNode.SelectNodes("ns:InnerResults/ns:UnitTestResult", nsm);

                    if (ddResults != null)
                    {
                        if (testResultInfo.Outcome == "Passed")
                        {
                            testResultInfo.Message = "[Data-Driven Test - all tests passed.]";
                        }
                        else
                        {
                            testResultInfo.Message = "[Data-Driven Test - one or more tests failed.]";
                        }

                        foreach (XmlNode ddNode in ddResults)
                        {
                            // Process data-driven test results
                            dataDrivenResultInfo = new TestResultInfo();

                            // Gather test-level properties
                            dataDrivenResultInfo.TestId = Utilities.GetAttributeValue(ddNode.Attributes, "testId");
                            dataDrivenResultInfo.ExecutionId = Utilities.GetAttributeValue(ddNode.Attributes, "executionId");
                            dataDrivenResultInfo.Outcome = Utilities.GetAttributeValue(ddNode.Attributes, "outcome");
                            dataDrivenResultInfo.IsDataDriven = (Utilities.GetAttributeValue(unitTestResultNode.Attributes, "resultType") == "DataDrivenTest");

                            // Gather method-level properties
                            dataDrivenResultInfo.ClassName = testResultInfo.ClassName;
                            dataDrivenResultInfo.MethodName = testResultInfo.MethodName;

                            // If the test failed, gather failure details
                            string outcome;
                            outcome = Utilities.GetAttributeValue(ddNode.Attributes, "outcome");
                            if (outcome == "Failed")
                            {
                                dataDrivenResultInfo.Message = ddNode.SelectSingleNode("ns:Output/ns:ErrorInfo/ns:Message", nsm).InnerText;
                                dataDrivenResultInfo.StackTrace = ddNode.SelectSingleNode("ns:Output/ns:ErrorInfo/ns:StackTrace", nsm).InnerText;
                            }

                            testResultInfo.DataDrivenResults.Add(dataDrivenResultInfo);
                        }
                    }
                }
                else
                {
                    // Retrieve results for failed non-data-driven test
                    if (testResultInfo.Outcome == "Failed")
                    {
                        testResultInfo.Message = unitTestResultNode.SelectSingleNode("ns:Output/ns:ErrorInfo/ns:Message", nsm).InnerText;
                        XmlNode stackTraceNode =
                            unitTestResultNode.SelectSingleNode("ns:Output/ns:ErrorInfo/ns:StackTrace", nsm);
                        if (stackTraceNode != null)
                        {
                            testResultInfo.StackTrace = stackTraceNode.InnerText;
                        }
                        else
                        {
                            testResultInfo.StackTrace = string.Empty;
                        }
                    }
                }

                testResult.TestResults.Add(testResultInfo);
            }

            testResult.MultiAssertReport = ApplyXslTransformation(testResult);

            return testResult;
        }

        /// <summary>
        /// Gets a list of the classes and methods from the unit test assembly.
        /// </summary>
        /// <param name="testId">The ID of the test to process.</param>
        /// <returns>A 2 level hierchial class/method list.</returns>
        public List<TestClass> GetTestMethods(int testId)
        {
            TestInfo testRow;
            testRow = GetTestRow(testId);

            if (!File.Exists(testRow.AssemblyPath))
            {
                return new List<TestClass>();
            }

            System.Reflection.Assembly targetAssembly = System.Reflection.Assembly.LoadFrom(testRow.AssemblyPath);

            // TODO implement using ReflectionOnly.  This requires calls to CustomAttributeData instead of GetCustomAttributes.
            ////System.AppDomain.CurrentDomain.ReflectionOnlyAssemblyResolve += new ResolveEventHandler(CurrentDomain_ReflectionOnlyAssemblyResolve);
            ////System.Reflection.Assembly targetAssembly = System.Reflection.Assembly.ReflectionOnlyLoad(testRow.AssemblyPath);
            ////System.Reflection.Assembly targetAssembly = System.Reflection.Assembly.ReflectionOnlyLoadFrom(testRow.AssemblyPath);

            var results =
                (
                    from testClass in targetAssembly.GetTypes()
                    where testClass.GetCustomAttributes(typeof(TestClassAttribute), true).Length > 0
                    select new TestClass()
                        {
                            Name = testClass.Name,
                            TestMethods = (
                                from testMethod in testClass.GetMethods()
                                where testMethod.GetCustomAttributes(typeof(TestMethodAttribute), true).Length > 0
                                select new TestMethod() { Name = testMethod.Name }
                                ).ToList()
                        }
                ).ToList();

            return results;
        }

        Assembly CurrentDomain_ReflectionOnlyAssemblyResolve(object sender, ResolveEventArgs args)
        {
            return System.Reflection.Assembly.ReflectionOnlyLoad(args.Name);
        }

        /// <summary>
        /// Runs the test for the test ID passed in.
        /// </summary>
        /// <param name="testId">The ID of the test to be ran.</param>
        /// <param name="testsToRun">A list of the test to run.  The expected format is 'Class.Method'.</param>
        /// <returns>
        /// The path of the generated test results file.
        /// </returns>
        public TestResultSummary RunTest(int testId, List<string> testsToRun, int timeoutSeconds)
        {
            string testResultsFilename = null;
            TestInfo testRow;
            string testFolder;
            string msTestArgs;
            List<DataSourceInfo> dataSources;
            TestResult testRunResults = null;
            TestResultSummary testRunResultsSummary = null;

            // Step 1: Retrieve test details
            testRow = GetTestRow(testId);

            // Step 2: Create a sub-folder within the working folder (for this test run)
            DataRow project = DataAccess.GetProject(testRow.ProjectId);
            string projectName = project["Name"].ToString();
            testFolder = Path.Combine(
                RtrSettings.WorkingFolder,
                string.Format("{0}.{1}.{2}", projectName, testRow.Name, DateTime.Now.ToString("MM-dd-yyyy_HH-mm-ss.fff")));
            Directory.CreateDirectory(testFolder);

            // Step 3: Copy the binaries (based on the assembly path) to the working folder
            CopyFiles(
                new DirectoryInfo(Path.GetDirectoryName(testRow.AssemblyPath)),
                new DirectoryInfo(testFolder));

            // Step 4: Copy the test version of the config file, if one is specified
            if (!string.IsNullOrEmpty(testRow.ConfigPath))
            {
                if (File.Exists(testRow.ConfigPath))
                {
                    File.Copy(testRow.ConfigPath, Path.Combine(testFolder, Path.GetFileName(testRow.ConfigPath)), true);
                }
                else
                {
                    throw new FileNotFoundException("The specified configuration file: " + testRow.ConfigPath + " could not be found.");
                }
            }

            // Step 5: Copy the data file(s) for this test
            if (File.Exists(testRow.ConfigPath))
            {
                dataSources = GetDataSources(testRow.ConfigPath);

                if ((dataSources != null) && (dataSources.Count > 0))
                {
                    // Download each file
                    foreach (DataSourceInfo dataSource in dataSources)
                    {
                        if (!string.IsNullOrEmpty(dataSource.DataSourceBaseUri))
                        {
                            DownloadFile(dataSource.DataSourceBaseUri, dataSource.DataSourcePath);
                        }
                    }
                }
            }

            // Step 6: Build command line for MSTest
            List<string> filesToSave = new List<string>();
            testResultsFilename = Path.Combine(testFolder, Path.GetFileName(testFolder) + ".trx");
            filesToSave.Add(testResultsFilename);
            string msTestRunLog = Path.Combine(testFolder, Path.GetFileName(testFolder) + ".txt");
            string testContainerFileName = Path.Combine(testFolder, Path.GetFileName(testRow.AssemblyPath));

            StringBuilder testsArgs = new StringBuilder();
            foreach (var tests in testsToRun)
            {
                testsArgs.Append(" /test:" + tests + " ");
            }

            msTestArgs = string.Format(
                "/nologo /testcontainer:\"{0}\" /resultsfile:\"{1}\" {3}",
                testContainerFileName,
                testResultsFilename,
                msTestRunLog,
                testsArgs.ToString());

            // Step 7: Run the test(s)
            ProcessStartInfo startInfo = new ProcessStartInfo(RtrSettings.MSTestPath, msTestArgs);
            startInfo.WindowStyle = ProcessWindowStyle.Hidden;
            startInfo.RedirectStandardOutput = true;
            startInfo.UseShellExecute = false;
            Process testProcess = Process.Start(startInfo);

            // Wait for the tests to complete before continuing - timeout after 10 minutes
            if (testProcess.WaitForExit(timeoutSeconds * 1000))
            {
                // Step 8: Parse the test results
                try
                {
                    testRunResultsSummary = ParseTestResultsSummary(testResultsFilename);
                    testRunResults = ParseTestResults(testResultsFilename);

                }
                catch (FileNotFoundException ex)
                {
                    throw new FileNotFoundException("The test results file: " + testResultsFilename + " was not found.", ex);
                }
            }
            else
            {
                try
                {
                    testProcess.Kill();

                }
                catch (Exception)
                {
                    // ignore any exception that occurred while trying to kill the process.
                }
                throw new TimeoutException("The tests have timed out during execution.  The tests were not completed.");
            }

            // Step 8: Remove temporary files
            testRunResults.MultiAssertResults.ForEach(
                result =>
                {
                    filesToSave.Add(result.FileName);
                });

            // Copy the save files to a safe directory
            string safeDirectory = RtrSettings.WorkingFolder;
            for (int i = 0; i < filesToSave.Count; i++)
            {
                string destinationFile = Path.Combine(safeDirectory, Path.GetFileName(filesToSave[i]));
                File.Copy(filesToSave[i], destinationFile);
                filesToSave[i] = destinationFile;
            }

            // Delete the working folder and all contents, and then recreate it
            Directory.Delete(testFolder, true);
            Directory.CreateDirectory(testFolder);

            // Move the save files back to the working folder
            for (int i = 0; i < filesToSave.Count; i++)
            {
                string destinationFile = Path.Combine(testFolder, Path.GetFileName(filesToSave[i]));
                File.Move(filesToSave[i], destinationFile);
                filesToSave[i] = destinationFile;
            }

            return testRunResultsSummary;
        }

        /// <summary>
        /// Gets the test result history.
        /// </summary>
        /// <param name="testId">The ID of the test.</param>
        /// <returns></returns>
        public List<TestResultSummary> GetTestResultHistory(int testId)
        {
            TestInfo testRow = GetTestRow(testId);

            DataRow project = DataAccess.GetProject(testRow.ProjectId);
            string projectName = project["Name"].ToString();

            string searchPattern = projectName + "." + testRow.Name + ".*.*";
            string[] testDirectories = System.IO.Directory.GetDirectories(RtrSettings.WorkingFolder, searchPattern);

            var results = new List<TestResultSummary>();

            foreach (var directory in testDirectories)
            {
                // Find the trx file in the results directory
                string[] testResultFiles = Directory.GetFiles(directory, "*.trx");

                // There should only be one trx file in the directory, so we choose the first item incase this isn't true
                if (testResultFiles.Length > 0)
                {
                    try
                    {
                        results.Add(ParseTestResultsSummary(testResultFiles[0]));
                    }
                    catch (Exception)
                    {
                        // we are just going to ignore any parse error that occurs.
                        // TODO handle parse errors more intelligently
                    }
                }
            }

            return results;
        }

        public TestResult GetTestResult(string path)
        {
            return ParseTestResults(path);
        }

        public int AddTest(int projectId, string name)
        {
            return 0;
        }

        public void DeleteTestResultHistoryItems(List<TestResultSummary> historyItemsToDelete)
        {
            foreach (TestResultSummary testResult in historyItemsToDelete)
            {
                string testPath = System.IO.Path.GetDirectoryName(testResult.Path);
                System.IO.Directory.Delete(testPath, true);
            }
        }

        private XmlDocument GetTestResultXmlDocument(string testResultsPath, out XmlNamespaceManager nsm)
        {
            XmlDocument doc = new XmlDocument();
            nsm = new XmlNamespaceManager(doc.NameTable);

            // Create namespace manager for the namespace used by MSTest.exe
            nsm.AddNamespace("ns", "http://microsoft.com/schemas/VisualStudio/TeamTest/2006");

            // Load the results document
            doc.Load(testResultsPath);
            return doc;
        }

        private string ApplyXslTransformation(TestResult testResult)
        {
            // Load up the XSL file.
            XsltSettings settings = new XsltSettings(true, false);
            XslCompiledTransform xslt = new XslCompiledTransform();
            string resource = GetResource("RemoteTestRunner.BusinessLogic.MultiAssertReport.MultiAssertReport.xsl");
            var reader = new StringReader(resource);
            var xmlReader = new System.Xml.XmlTextReader(reader);
            xslt.Load(xmlReader, settings, null);

            // Prepare the output stream.
            StringWriter stringWriter = new StringWriter();
            var writer = new System.Xml.XmlTextWriter(stringWriter);

            // Transform the raw data and return the result.
            // Create an XML stream that looks like <Files><File>c:\...</File></Files>
            XElement externalFiles = new XElement(
                "Files",
                from f in testResult.MultiAssertResults
                select new XElement("File", f.FileName));

            XmlDocument doc = new XmlDocument();
            doc.LoadXml(externalFiles.ToString());
            XmlNode node;
            node = doc;

            xslt.Transform(node, writer);
            return stringWriter.ToString();
        }

        #endregion

        private static string GetResource(string resourceName)
        {
            Assembly assembly = Assembly.GetExecutingAssembly();
            TextReader textReader = new StreamReader(assembly.GetManifestResourceStream(resourceName));
            string result = textReader.ReadToEnd();
            textReader.Close();

            return result;
        }

        #region Private Methods

        /// <summary>
        /// Recursively copies all files and folders from the source folder to the target
        /// folder.
        /// </summary>
        /// <param name="source">The source folder.</param>
        /// <param name="target">The target folder.</param>
        private void CopyFiles(DirectoryInfo source, DirectoryInfo target)
        {
            // Check if the target directory exists, if not, create it.
            if (Directory.Exists(target.FullName) == false)
            {
                Directory.CreateDirectory(target.FullName);
            }

            // Copy each file into it’s new directory.
            foreach (FileInfo fi in source.GetFiles())
            {
                fi.CopyTo(Path.Combine(target.ToString(), fi.Name), true);
            }

            // Copy each subdirectory using recursion.
            foreach (DirectoryInfo diSourceSubDir in source.GetDirectories())
            {
                DirectoryInfo nextTargetSubDir =
                    target.CreateSubdirectory(diSourceSubDir.Name);
                CopyFiles(diSourceSubDir, nextTargetSubDir);
            }
        }

        /// <summary>
        /// Downloads the specified file.
        /// </summary>
        /// <param name="requestUriString">The URI of the file to be downloaded.</param>
        /// <param name="desinationPath">The desination path.</param>
        /// <returns><c>true</c> if the download is successful; otherwise, <c>false</c>.</returns>
        private bool DownloadFile(string requestUriString, string desinationPath)
        {
            bool success = true;

            try
            {
                WebClient client = new WebClient();

                client.UseDefaultCredentials = true;
                client.DownloadFile(requestUriString, HttpUtility.UrlDecode(desinationPath));
            }
            catch
            {
                success = false;
            }

            return success;
        }

        #endregion

    }
}
