﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Windows.Forms;
using System.Xml;
using RemoteTestRunner.BusinessLogic;
using RemoteTestRunner.FormsUI.Properties;
using RemoteTestRunner.ServiceFacade;
using RemoteTestRunner.WCFService;

namespace RemoteTestRunner.FormsUI
{
    public partial class Runner : Form
    {
        #region Private Attributes

        private IRTRService _serviceFacade;
        private DataView _testView = new DataView();
        private DataSet _testData;
        private DataRow _currentTestRow;
        private TestResult _testResult;
        private static LongRunning _runStatusForm;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Runner"/> class.
        /// </summary>
        public Runner()
        {
            InitializeComponent();

            // Setup the appropriate test runner facade (i.e. will we be running from the local
            // client or the server?
            InitializeServiceFacade();
        }

        #endregion

        #region Event Handlers

        /// <summary>
        /// Handles the Load event of the Runner control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void Runner_Load(object sender, EventArgs e)
        {
            if (!File.Exists(RtrSettings.DataPath))
            {
                MessageBox.Show(
                    "You must select a data file to work with before continuing.",
                    "Select Data File",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Information);

                DisplayOptionsDialog();
            }
        }

        /// <summary>
        /// Opens an existing test results file for viewing.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void mnuFileOpen_Click(object sender, EventArgs e)
        {
            OpenTestResultsFile();
        }

        /// <summary>
        /// Sends an e-mail containing the details for any failed or inconclusive tests.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void mnuFileSendEmail_Click(object sender, EventArgs e)
        {
            RtrLib.StartEmail(_testResult);
        }

        /// <summary>
        /// Handles the Click event of the mnuFileExit control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void mnuFileExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Handles the Click event of the mnuToolsConfigureTests control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void mnuToolsConfigureTests_Click(object sender, EventArgs e)
        {
            using (TestAdmin testAdmin = new TestAdmin(_serviceFacade))
            {
                if (testAdmin.ShowDialog(this) == DialogResult.OK)
                {
                    LoadProjects();
                }
            }
        }

        /// <summary>
        /// Handles the Click event of the mnuToolsConfigureTestProjects control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void mnuToolsConfigureTestProjects_Click(object sender, EventArgs e)
        {
            using (ProjectAdmin projectAdmin = new ProjectAdmin(_serviceFacade))
            {
                projectAdmin.ShowDialog(this);
            }
        }

        /// <summary>
        /// Hides/shows the inputs panel.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void mnuToolsHideShowInputs_Click(object sender, EventArgs e)
        {
            splitContainerSub.Panel1Collapsed = !splitContainerSub.Panel1Collapsed;
            splitContainerMain.Panel1Collapsed = splitContainerSub.Panel1Collapsed;

            toolToggleInputs.Checked = !splitContainerSub.Panel1Collapsed;
            mnuToolsHideShowInputs.Checked = !splitContainerSub.Panel1Collapsed;
        }

        /// <summary>
        /// Handles the Click event of the mnuToolsOptions control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void mnuToolsOptions_Click(object sender, EventArgs e)
        {
            DisplayOptionsDialog();
        }

        /// <summary>
        /// Handles the Click event of the mnuTestRun control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void mnuTestRun_Click(object sender, EventArgs e)
        {
            RunTest();
        }

        /// <summary>
        /// Displays the selected data source.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ViewDataSource_Click(object sender, EventArgs e)
        {
            string filename;

            if ((sender != null) && (sender is ToolStripMenuItem))
            {
                filename = ((ToolStripMenuItem)sender).Tag.ToString();

                if (File.Exists(filename))
                {
                    // Launch the data source file in the default application for the file type
                    Process.Start(filename);
                }
                else
                {
                    // The file was not found
                    MessageBox.Show(
                        "The data source could not be opened.  Please ensure the file exists.",
                        "File Not Found",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Warning);
                }
            }
        }

        /// <summary>
        /// Handles the Click event of the mnuHelpAbout control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void mnuHelpAbout_Click(object sender, EventArgs e)
        {
            using (About aboutDlg = new About())
            {
                aboutDlg.ShowDialog(this);
            }
        }

        /// <summary>
        /// Displays the help file.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void mnuHelpTopics_Click(object sender, EventArgs e)
        {
            Process.Start("RTRDocs.chm");
        }

        /// <summary>
        /// Handles the SelectionChangeCommitted event of the cboProjects control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void cboProjects_SelectionChangeCommitted(object sender, EventArgs e)
        {
            BindTestList();
        }

        /// <summary>
        /// Handles the SelectionChangeCommitted event of the cboTests control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void cboTests_SelectionChangeCommitted(object sender, EventArgs e)
        {
            TestSelected();
        }

        /// <summary>
        /// Copies details for the selected test results to the Clipboard.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void mnuTestCopyTestDetails_Click(object sender, EventArgs e)
        {
            string text = string.Empty;
            int row;

            foreach (ListViewItem lvItem in lvResults.SelectedItems)
            {
                foreach (TestResultInfo item in _testResult.TestResults)
                {
                    if (item.ExecutionId == lvItem.Tag.ToString())
                    {
                        text += item.ToString() + "\r\n\r\n========================\r\n\r\n";

                        break;
                    }
                    else
                    { // Search in any data-driven results
                        row = 0;

                        foreach (TestResultInfo ddItem in item.DataDrivenResults)
                        {
                            row++;

                            if (ddItem.ExecutionId == lvItem.Tag.ToString())
                            {
                                text += string.Format("[row: {0}]", row) +
                                    ddItem.ToString() + "\r\n\r\n========================\r\n\r\n";

                                break;
                            }
                        }
                    }
                }
            }

            Clipboard.SetText(text);
        }

        /// <summary>
        /// Enables/disables form options based upon the current state.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.ComponentModel.CancelEventArgs"/> instance containing the event data.</param>
        private void contextMenu_Opening(object sender, System.ComponentModel.CancelEventArgs e)
        {
            UpdateOptions();
        }

        /// <summary>
        /// Enables/disables form options based upon the current state.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void mnuTest_DropDownOpening(object sender, EventArgs e)
        {
            UpdateOptions();
        }

        /// <summary>
        /// Validates items that are attempting to be "dragged-n-dropped" on the ListView control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.DragEventArgs"/> instance containing the event data.</param>
        /// <remarks>The item being dragged must be a file with a ".trx" extension.</remarks>
        private void lvResults_DragEnter(object sender, DragEventArgs e)
        {
            string filename = null;
            bool dropAllowed = false;

            if (e.Data.GetDataPresent("FileNameW"))
            {
                filename = ((string[])e.Data.GetData("FileNameW"))[0];

                dropAllowed = (string.Compare(Path.GetExtension(filename), ".trx", true) == 0);
            }

            e.Effect = dropAllowed ? DragDropEffects.Copy : DragDropEffects.None;
        }

        /// <summary>
        /// Opens the item being "dragged-n-dropped" on the ListView control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.DragEventArgs"/> instance containing the event data.</param>
        /// <remarks>The item being dropped must be a file with a ".trx" extension.</remarks>
        private void lvResults_DragDrop(object sender, DragEventArgs e)
        {
            DisplayResults(((string[])e.Data.GetData("FileNameW"))[0]);
        }

        /// <summary>
        /// Handles the SelectedValueChanged event of the lstRunHistory control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void lstRunHistory_SelectedValueChanged(object sender, EventArgs e)
        {
            if (lstRunHistory.SelectedValue != null)
            {
                TestResultSummary selectedSummary = lstRunHistory.SelectedValue as TestResultSummary;
                _testResult = _serviceFacade.GetTestResult(selectedSummary.Path);
                DisplayResults();
            }
        }

        /// <summary>
        /// Handles the Click event of the deleteToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void deleteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Delete selected Test Run History items?", "Delete Items", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
            {
                DeleteTestRunHistoryItems();
            }
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Get the appropriate service facade based on whether we're planning on running
        /// the tests locally or on a server (e.g. build server).
        /// </summary>
        private void InitializeServiceFacade()
        {
            if (RtrSettings.UseRunnerService)
            {
                // Run on server
                _serviceFacade = ServiceFactory.GetServiceObject( 
                    RtrSettings.RunnerServiceUrl,
                    RtrSettings.TestTimeoutSeconds);
            }
            else
            {
                // Run on local client
                _serviceFacade = ServiceFactory.GetServiceObject(ServiceType.ClientService);
            }

            LoadProjects();
        }

        /// <summary>
        /// Set up the form after a test is selected.
        /// </summary>
        private void TestSelected()
        {
            ClearResults();

            // Locate the row for the newly created test
            _currentTestRow = _testData.Tables["Test"].Select("ID = " + cboTests.SelectedValue.ToString())[0];

            // Display the details
            lblAssemblyPath.Text = _currentTestRow["AssemblyPath"].ToString();
            lblConfigurationPath.Text = _currentTestRow["ConfigPath"].ToString();

            FillTestClassesAndMethods();
            LoadPreviousTestRuns();

            SetupDataSourceMenu();
            UpdateOptions();
        }

        /// <summary>
        /// Loads the projects.
        /// </summary>
        private void LoadProjects()
        {
            cboTests.DataSource = null;
            tvwClassesAndMethods.Nodes.Clear();
            lstRunHistory.DataSource = null;
            ClearResults();

            _testData = _serviceFacade.GetTestData();
            BindProjectList();
            UpdateOptions();
        }

        /// <summary>
        /// Configures the data binding for this form.
        /// </summary>
        private void BindProjectList()
        {
            cboProjects.DataSource = _testData.Tables["Project"];// (DataTable)_serviceFacade.GetTestProjects();
            cboProjects.DisplayMember = "Name";
            cboProjects.ValueMember = "ID";

            if (cboProjects.Items.Count == 1)
            {
                cboProjects.SelectedIndex = 0;
                BindTestList();
            }
            else
            {
                cboProjects.SelectedIndex = -1;
            }
        }

        /// <summary>
        /// Binds the test data source.
        /// </summary>
        private void BindTestList()
        {
            _testView =
                new DataView(
                    _testData.Tables["Test"],
                    "ProjectId = " + cboProjects.SelectedValue.ToString(),
                    string.Empty,
                    DataViewRowState.CurrentRows);

            cboTests.DataSource = _testView;
            cboTests.DisplayMember = "Name";
            cboTests.ValueMember = "ID";

            if (cboTests.Items.Count == 1)
            {
                cboTests.SelectedIndex = 0;
                TestSelected();
            }
            else
            {
                cboTests.SelectedIndex = -1;
            }

            UpdateOptions();
        }

        /// <summary>
        /// Enables/disables form options based upon the current state.
        /// </summary>
        private void UpdateOptions()
        {
            mnuTestRun.Enabled = cboTests.SelectedIndex >= 0;
            toolRunTest.Enabled = mnuTestRun.Enabled;

            mnuTestCopyTestDetails.Enabled = (lvResults.SelectedItems.Count > 0);
            ctxCopyTestDetails.Enabled = mnuTestCopyTestDetails.Enabled;

            ctxDeleteTestRunHistoryItem.Enabled = lstRunHistory.SelectedItems.Count > 0;
            mnuDeleteTestRunHistoryItem.Enabled = lstRunHistory.SelectedItems.Count > 0;

            mnuFileSendEmail.Enabled = _testResult != null;
            toolSendNotification.Enabled = _testResult != null;

            mnuTestViewDataSource.Enabled = cboTests.SelectedIndex >= 0;
        }

        private delegate TestResultSummary RunTestDelegate(int testId, List<string> testsToRun, int timeoutSeconds);

        /// <summary>
        /// Runs the selected test.
        /// </summary>
        private void RunTest()
        {
            if (MessageBox.Show("Run selected test?", "Run Test", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                Cursor.Current = Cursors.WaitCursor;

                try
                {
                    RunTestDelegate runTestDelegate = new RunTestDelegate(_serviceFacade.RunTest);
                    runTestDelegate.BeginInvoke(
                        (int)cboTests.SelectedValue, GetCheckedNodes(), RtrSettings.TestTimeoutSeconds, RunTestCompleted, runTestDelegate);

                    _runStatusForm = new LongRunning();
                    _runStatusForm.ProcessDescription = "Running test(s)...";
                    _runStatusForm.ShowDialog(this);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(
                        ex.ToString(), "Test Run Message", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }
        }

        /// <summary>
        /// Completion method for async RunTest.
        /// </summary>
        /// <param name="ar">The async result.</param>
        private void RunTestCompleted(IAsyncResult ar)
        {
            if (InvokeRequired)
            {
                AsyncCallback callback = new AsyncCallback(RunTestCompleted);
                Invoke(callback, ar);
                return;
            }

            _runStatusForm.Hide();
            _runStatusForm.Refresh();
            _runStatusForm.Close();
            RunTestDelegate runTestDelegate = ar.AsyncState as RunTestDelegate;

            try
            {
                TestResultSummary summary = null;
                try
                {
                    summary = runTestDelegate.EndInvoke(ar);
                }
                catch (Exception ex)
                {
                    summary = null;
                    MessageBox.Show(
                        ex.ToString(), "Test Run Message", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }

                if (summary != null)
                {
                    // Save the run that just completed for selecting.
                    string savedId = summary.TestRunId;

                    // re-populate the test runs now that we have a new one.
                    // TODO come up with a better way to populate test runs.  Maybe use a file watcher (this might not work for the remote version).
                    LoadPreviousTestRuns();

                    // Select the run that just completed.
                    foreach (TestResultSummary result in lstRunHistory.Items)
                    {
                        if (result.TestRunId == savedId)
                        {
                            lstRunHistory.SelectedItem = result;
                            break;
                        }
                    }
                }
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }
        }

        /// <summary>
        /// Fills the test classes and methods.
        /// </summary>
        private void FillTestClassesAndMethods()
        {
            // Retrieve the classes and methods from the assembly
            IList<TestClass> testMethods;
            testMethods = _serviceFacade.GetTestMethods((int)cboTests.SelectedValue);

            // Fill the selection tree view with the classes and methods
            if (testMethods != null)
            {
                tvwClassesAndMethods.Nodes.Clear();

                // Add a root node that is checked by default.
                TreeNode rootNode = tvwClassesAndMethods.Nodes.Add("All Classes and Methods");
                //rootNode.Checked = true;

                // Add the hierarchial classes and methods
                foreach (var testClass in testMethods)
                {
                    TreeNode classNode = rootNode.Nodes.Add(testClass.Name);
                    foreach (var testMethod in testClass.TestMethods)
                    {
                        TreeNode methodNode = classNode.Nodes.Add(testMethod.Name);
                    }
                }

                tvwClassesAndMethods.ExpandAll();
                rootNode.EnsureVisible();
            }
        }

        /// <summary>
        /// Fills the previous test runs.
        /// </summary>
        private void LoadPreviousTestRuns()
        {
            IList<TestResultSummary> testResults;
            testResults = _serviceFacade.GetTestResultHistory((int)cboTests.SelectedValue);
            // Sort the results by start time descending.
            IEnumerable<TestResultSummary> orderedResults = testResults.OrderBy(result => result.TestStartTime).Reverse();
            if (orderedResults.Count() > 0)
            {
                lstRunHistory.DisplayMember = "TestRunName";
                lstRunHistory.DataSource = new BindingSource(orderedResults, null);
            }
            else
            {
                lstRunHistory.DataSource = null;
            }
        }

        /// <summary>
        /// Gets the checked nodes.
        /// </summary>
        /// <returns>An IList of all the checked nodes.</returns>
        private List<string> GetCheckedNodes()
        {
            var returnList = new List<string>();
            foreach (TreeNode root in tvwClassesAndMethods.Nodes)
            {
                if (root.Checked)
                {
                    // root is checked.  Return an empty list so all tests will run.
                }
                else
                {
                    foreach (TreeNode classNode in root.Nodes)
                    {
                        if (classNode.Checked)
                        {
                            // Add whole class
                            returnList.Add(classNode.Text);
                        }
                        else
                        {
                            foreach (TreeNode methodNode in classNode.Nodes)
                            {
                                if (methodNode.Checked)
                                {
                                    returnList.Add(classNode.Text + "." + methodNode.Text);
                                }
                            }
                        }
                    }
                }
            }

            return returnList;
        }

        /// <summary>
        /// Setups the data source menu.
        /// </summary>
        private void SetupDataSourceMenu()
        {
            ToolStripMenuItem menuItem;

            // Clear out the sub-menus
            mnuTestViewDataSource.DropDown.Items.Clear();
            ctxViewDataSource.DropDown.Items.Clear();

            TestInfo testInfo = _serviceFacade.GetTestRow((int)cboTests.SelectedValue);
            List<DataSourceInfo> dsInfo = _serviceFacade.GetDataSources(testInfo.ConfigPath);

            if (dsInfo != null)
            {
                foreach (DataSourceInfo item in dsInfo)
                {
                    // Populate Test->View Data Source menu
                    menuItem = new ToolStripMenuItem(item.ConnectionStringName, null, new EventHandler(ViewDataSource_Click), item.Name);
                    menuItem.Tag = item.DataSourcePath;
                    mnuTestViewDataSource.DropDown.Items.Add(menuItem);

                    // Populate context menu
                    menuItem = new ToolStripMenuItem(item.ConnectionStringName, null, new EventHandler(ViewDataSource_Click), item.Name);
                    menuItem.Tag = item.DataSourcePath;
                    ctxViewDataSource.DropDown.Items.Add(menuItem);
                }
            }

            // Disable the menu option if there are no data sources
            mnuTestViewDataSource.Enabled = (mnuTestViewDataSource.DropDown.Items.Count > 0);
            ctxViewDataSource.Enabled = (ctxViewDataSource.DropDown.Items.Count > 0);
        }

        /// <summary>
        /// Clears the results.
        /// </summary>
        private void ClearResults()
        {
            lvResults.Items.Clear();
            var presenter = new Presenters.DetailedResultsPresenter(detailResults1, null);
            presenter.Start();

            lblTotalCount.Text = "0";
            lblTotalExecuted.Text = "0";
            lblTotalPassed.Text = "0";
            lblTotalFailed.Text = "0";
            lblTotalTimedOut.Text = "0";
            lblTotalInconclusive.Text = "0";

            SetStatusMessage("Ready");
        }

        /// <summary>
        /// Loads the results XML file and displays the formatted test results.
        /// </summary>
        private void DisplayResults()
        {
            DisplayResults(_testResult);
        }

        /// <summary>
        /// Displays the test results file.
        /// </summary>
        /// <param name="testResultsPath">The path to the test results file to display.</param>
        private void DisplayResults(string testResultsPath)
        {
            RtrLib rtrLib = new RtrLib();
            _testResult = rtrLib.ParseTestResults(testResultsPath);

            if (_testResult == null)
            {
                MessageBox.Show(
                    "The selected file is not a valid test results file.",
                    "Invalid File Format",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Warning);
            }
            else
            {
                DisplayResults();
            }
        }

        /// <summary>
        /// Loads the results XML file and displays the formatted test results.
        /// </summary>
        /// <param name="testResult">The test results to be displayed.</param>
        private void DisplayResults(TestResult testResult)
        {
            Dictionary<string, int> outcome = new Dictionary<string, int>();
            ListViewItem lvItem;
            int ddTestCount;

            // Display the filename
            this.Text = string.Format("Remote Test Runner - [{0}]", Path.GetFileNameWithoutExtension(testResult.Path));

            outcome.Add("Passed", 0);
            outcome.Add("Failed", 0);
            outcome.Add("Inconclusive", 0);
            outcome.Add("Skipped", 0);

            lvResults.Items.Clear();

            foreach (TestResultInfo resultInfo in testResult.TestResults)
            {
                lvItem = new ListViewItem(
                    new string[]
                    {
                        resultInfo.MethodName,
                        resultInfo.Message
                    },
                    lvResults.Groups[resultInfo.Outcome]);

                // Store the Execution ID
                lvItem.Tag = resultInfo.ExecutionId;

                // Display an icon for each test
                switch (resultInfo.Outcome)
                {
                    case "Passed":
                    case "Failed":
                        lvItem.ImageKey = resultInfo.Outcome;
                        break;
                    default:
                        lvItem.ImageKey = "Inconclusive";
                        break;
                }

                // Display the detailed results in the ToolTip
                lvItem.ToolTipText = resultInfo.ToString();

                // If the test failed, display the item in a different color/weight
                if (resultInfo.Outcome == "Failed")
                {
                    lvItem.ForeColor = Color.Red;
                }

                lvResults.Items.Add(lvItem);

                if (resultInfo.IsDataDriven)
                {
                    ddTestCount = 0;

                    // Add sub-rows for data-driven test results
                    foreach (TestResultInfo ddResultInfo in resultInfo.DataDrivenResults)
                    {
                        ddTestCount++;

                        lvItem = new ListViewItem(
                            new string[]
                            {
                                string.Format("     [row: {0}]", ddTestCount), //ddResultInfo.MethodName,
                                ddResultInfo.Message
                            },
                            lvResults.Groups[resultInfo.Outcome]);

                        // Store the Execution ID
                        lvItem.Tag = ddResultInfo.ExecutionId;

                        // Display an icon for each test
                        lvItem.ImageKey = ddResultInfo.Outcome;

                        // Display the detailed results in the ToolTip
                        lvItem.ToolTipText = ddResultInfo.ToString();

                        // If the test failed, display the item in a different color/weight
                        if (ddResultInfo.Outcome == "Failed")
                        {
                            lvItem.ForeColor = Color.Red;
                        }

                        lvResults.Items.Add(lvItem);
                    }
                }
            }

            // Display summary results
            lblTotalCount.Text = testResult.TotalCount.ToString("N0");
            lblTotalExecuted.Text = testResult.TotalExecuted.ToString("N0");
            lblTotalPassed.Text = testResult.TotalPassed.ToString("N0");
            lblTotalFailed.Text = testResult.TotalFailed.ToString("N0");
            lblTotalTimedOut.Text = testResult.TotalTimedOut.ToString("N0");
            lblTotalInconclusive.Text = testResult.TotalInconclusive.ToString("N0");

            // Set status message
            if (testResult.TotalExecuted == testResult.TotalPassed)
            {
                SetStatusMessage("All tests passed", Color.LightGreen);
            }
            else if ((testResult.TotalFailed > 0) || (testResult.TotalTimedOut > 0))
            {
                SetStatusMessage("One or more tests failed and/or timed out", Color.LightCoral);
            }
            else if (outcome["Inconclusive"] > 0)
            {
                SetStatusMessage("One or more tests were inconclusive", Color.LightYellow);
            }

            var presenter = new Presenters.DetailedResultsPresenter(detailResults1, testResult);
            presenter.Start();
        }

        /// <summary>
        /// Sets the status message.
        /// </summary>
        /// <param name="message">The message to be displayed.</param>
        private void SetStatusMessage(string message)
        {
            SetStatusMessage(message, SystemColors.Control);
        }

        /// <summary>
        /// Sets the status message.
        /// </summary>
        /// <param name="message">The message to be displayed.</param>
        /// <param name="backColor">The color to set the status bar background color to.</param>
        private void SetStatusMessage(string message, Color backColor)
        {
            ssStatus.Items[0].Text = message;

            ssStatus.BackColor = backColor;

            Application.DoEvents();
        }

        /// <summary>
        /// Displays the options dialog.
        /// </summary>
        private void DisplayOptionsDialog()
        {
            bool useRunnerServiceOrig = RtrSettings.UseRunnerService;
            string serviceUrlOrig = RtrSettings.RunnerServiceUrl;

            using (Options optionsDlg = new Options())
            {
                if (optionsDlg.ShowDialog(this) == DialogResult.OK)
                {
                    // If we've switched from client to server (or vice-versa) or, if we are running
                    // against the server and the URL has changed, then reinitialize the service
                    // facade
                    if ((useRunnerServiceOrig != RtrSettings.UseRunnerService) ||
                        (((useRunnerServiceOrig == true) &&
                          !string.Equals(serviceUrlOrig, RtrSettings.RunnerServiceUrl))))
                    {
                        // Setup the appropriate test runner facade
                        InitializeServiceFacade();
                    }

                }
            }
        }

        /// <summary>
        /// Opens an existing test results file.
        /// </summary>
        private void OpenTestResultsFile()
        {
            openFileDialog1.CheckFileExists = true;
            openFileDialog1.CheckPathExists = true;
            openFileDialog1.DefaultExt = ".trx";
            openFileDialog1.Filter = "Test Result Files (*.trx)|*.trx|All Files (*.*)|*.*";
            openFileDialog1.FilterIndex = 0;
            openFileDialog1.InitialDirectory = RtrSettings.WorkingFolder;
            openFileDialog1.Multiselect = false;
            openFileDialog1.Title = "Select Test Result File";

            if (openFileDialog1.ShowDialog(this) == DialogResult.OK)
            {
                DisplayResults(openFileDialog1.FileName);
            }
        }

        /// <summary>
        /// Deletes the test run history items.
        /// </summary>
        private void DeleteTestRunHistoryItems()
        {
            if (lstRunHistory.SelectedItems.Count > 0)
            {
                var historyToDelete = new List<TestResultSummary>();
                foreach (var selectedItem in lstRunHistory.SelectedItems)
                {
                    historyToDelete.Add(selectedItem as TestResultSummary);
                }

                _serviceFacade.DeleteTestResultHistoryItems(historyToDelete);

                LoadPreviousTestRuns();
            }
        }

        #endregion
    }
}
