﻿using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Windows.Forms;
using RemoteTestRunner.BusinessLogic;
using RemoteTestRunner.ServiceFacade;
using RemoteTestRunner.WCFService;

namespace RemoteTestRunner.FormsUI
{
    public partial class TestAdmin : Form
    {
        #region Constants

        private string defaultTestName = "<New Test>";

        #endregion

        #region Private Attributes

        private IRTRService _serviceFacade;
        private DataView _testView = new DataView();
        private DataSet _testData;
        private DataTable _currentTests;
        private DataRow _currentTestRow;
        private bool _isLoading = true;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="TestAdmin"/> class.
        /// </summary>
        public TestAdmin(IRTRService serviceFacade)
        {
            InitializeComponent();

            // Use the appropriate service facade based on whether we're planning on running
            // the tests locally or on a server (e.g. build server)
            _serviceFacade = serviceFacade;

            // Get a DataSet containing all project/test configuration data
            _testData = _serviceFacade.GetTestData();

            ConfigureProjectDataBinding();

            _isLoading = false;

            UpdateOptions();
        }

        #endregion

        #region Event Handlers

        /// <summary>
        /// Initializes the form.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void TestAdmin_Load(object sender, EventArgs e)
        {
            // Force the first test to be selected and the "SelectedIndexChanged" event to be fired
            if (lstTests.Items.Count > 0)
            {
                lstTests.SelectedIndex = -1;
                lstTests.SelectedIndex = 0;
            }
        }

        /// <summary>
        /// Saves any changes and closes the form.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void btnOK_Click(object sender, EventArgs e)
        {
            _serviceFacade.SaveTestData(_testData);

            this.DialogResult = DialogResult.OK;
            this.Close();
        }

        #endregion

        #region Miscellaneous Methods

        /// <summary>
        /// Enables/disables form options based upon the current state.
        /// </summary>
        private void UpdateOptions()
        {
            // Test Controls
            lstTests.Enabled = (cboProjects.SelectedIndex >= 0);
            txtTestName.Enabled = (lstTests.SelectedIndex >= 0);
            txtAssemblyPath.Enabled = txtTestName.Enabled;
            txtConfigurationPath.Enabled = txtAssemblyPath.Enabled;
            btnAddTest.Enabled = lstTests.Enabled;
            btnDeleteTest.Enabled = txtAssemblyPath.Enabled;
            btnBrowseAssembly.Enabled = txtAssemblyPath.Enabled;
            btnBrowseConfigPath.Enabled = txtConfigurationPath.Enabled;
        }

        #endregion

        #region Project Methods/Event Handlers

        /// <summary>
        /// Configures the project data binding for this form.
        /// </summary>
        private void ConfigureProjectDataBinding()
        {
            // Projects
            cboProjects.DataSource = _testData.Tables["Project"];
            cboProjects.DisplayMember = "Name";
            cboProjects.ValueMember = "ID";

            if (cboProjects.Items.Count > 0)
            {
                BindTestList();
            }
        }

        /// <summary>
        /// Re-binds the test data source when the project selection has changed.
        /// </summary>
        private void cboProjects_SelectionChangeCommitted(object sender, EventArgs e)
        {
            BindTestList();

            UpdateOptions();
        }

        /// <summary>
        /// Binds the test data source.
        /// </summary>
        private void BindTestList()
        {
            _currentTests = _testData.Tables["Test"];

            _testView = new DataView(_currentTests, "ProjectId = " + cboProjects.SelectedValue.ToString(), string.Empty, DataViewRowState.CurrentRows);

            lstTests.DataSource = _testView;
            lstTests.DisplayMember = "Name";
            lstTests.ValueMember = "ID";

            if (lstTests.Items.Count > 0)
            {
                lstTests.SelectedIndex = 0;
            }
        }

        #endregion

        #region Test Methods/Event Handlers

        /// <summary>
        /// Adds a new test item.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void btnAddTest_Click(object sender, EventArgs e)
        {
            //// Add a new test and get back the new test ID
            int newTestId = (int)_testData.Tables["Test"].Rows.Add(
                null, 
                defaultTestName, 
                string.Empty, 
                string.Empty, 
                Environment.UserName, 
                DateTime.Now, 
                Environment.UserName, 
                DateTime.Now, 
                (int)cboProjects.SelectedValue)["ID"];

            // Locate the row for the newly created test
            _currentTestRow = _currentTests.Select("ID = " + newTestId.ToString())[0];

            DisplayTestDetails((int)_currentTestRow["ID"]);
            UpdateOptions();
        }

        /// <summary>
        /// Deletes the selected test item.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void btnDeleteTest_Click(object sender, EventArgs e)
        {
            if (lstTests.SelectedIndex >= 0)
            {
                if (MessageBox.Show(
                    string.Format("Delete the test '{0}'?", lstTests.Text),
                    "Delete Test", 
                    MessageBoxButtons.YesNo, 
                    MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    _serviceFacade.DeleteTest((int)lstTests.SelectedValue);

                    UpdateOptions();
                }
            }
        }

        /// <summary>
        /// Displays the details for the specified test ID.
        /// </summary>
        /// <param name="testId">The test ID to display details for.</param>
        private void DisplayTestDetails(int testId)
        {
            for (int index = 0; index < lstTests.Items.Count; index++)
            {
                if (((int)((DataRowView)lstTests.Items[index])["ID"]) == testId)
                {
                    int oldIndex = lstTests.SelectedIndex;

                    lstTests.SelectedIndex = index;

                    if (oldIndex == index)
                    {
                        DisplayTestDetails();
                    }

                    break;
                }
            }
        }

        /// <summary>
        /// Displays the details for the currently selected test.
        /// </summary>
        private void DisplayTestDetails()
        {
            txtTestName.Text = _currentTestRow["Name"].ToString();
            txtAssemblyPath.Text = _currentTestRow["AssemblyPath"].ToString();
            txtConfigurationPath.Text = _currentTestRow["ConfigPath"].ToString();
        }

        /// <summary>
        /// Displays details when a test is clicked.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void lstTests_SelectedIndexChanged(object sender, EventArgs e)
        {
            if ((lstTests.SelectedIndex >= 0) && (!_isLoading))
            {
                // Locate the row for the newly created test
                _currentTestRow = _currentTests.Select("ID = " + lstTests.SelectedValue.ToString())[0];

                DisplayTestDetails();

                LoadDataSources();

                UpdateOptions();
            }
        }

        /// <summary>
        /// Updates the <see cref="DataRow"/> when the text has been modified.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void txtTestName_TextChanged(object sender, EventArgs e)
        {
            if (_currentTestRow != null)
            {
                _currentTestRow["Name"] = txtTestName.Text;
            }
        }

        /// <summary>
        /// Updates the <see cref="DataRow"/> when the text has been modified.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void txtAssemblyPath_TextChanged(object sender, EventArgs e)
        {
            if (_currentTestRow != null)
            {
                _currentTestRow["AssemblyPath"] = txtAssemblyPath.Text;
            }
        }

        /// <summary>
        /// Updates the <see cref="DataRow"/> when the text has been modified.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void txtConfigurationPath_TextChanged(object sender, EventArgs e)
        {
            if (_currentTestRow != null)
            {
                _currentTestRow["ConfigPath"] = txtConfigurationPath.Text;
            }
        }

        /// <summary>
        /// Browses for the test assembly file.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void btnBrowseAssembly_Click(object sender, EventArgs e)
        {
            // Configure the "Open File" common dialog
            openFileDialog1.CheckFileExists = true;
            openFileDialog1.CheckPathExists = true;
            openFileDialog1.Filter = ".NET Assemblies (*.dll)|*.dll|All Files (*.*)|*.*";
            openFileDialog1.FilterIndex = 0;
            openFileDialog1.Multiselect = false;
            openFileDialog1.Title = "Select Unit Test Assmebly";

            if (openFileDialog1.ShowDialog(this) == DialogResult.OK)
            {
                txtAssemblyPath.Text = openFileDialog1.FileName;

                // If no configuration file has been specified, then attempt to locate the
                // default app.Config file
                if (txtConfigurationPath.Text.Trim().Length == 0)
                {
                    string appConfigPath = txtAssemblyPath.Text + ".Config";

                    if (File.Exists(appConfigPath))
                    {
                        txtConfigurationPath.Text = appConfigPath;
                    }
                }
            }
        }

        /// <summary>
        /// Browses for the test assembly configuration file.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void btnBrowseConfigPath_Click(object sender, EventArgs e)
        {
            // Configure the "Open File" common dialog
            openFileDialog1.CheckFileExists = true;
            openFileDialog1.CheckPathExists = true;
            openFileDialog1.Filter = "Config Files (*.config)|*.config|All Files (*.*)|*.*";
            openFileDialog1.FilterIndex = 0;
            openFileDialog1.Multiselect = false;
            openFileDialog1.Title = "Select Configuration File";

            if (openFileDialog1.ShowDialog(this) == DialogResult.OK)
            {
                txtConfigurationPath.Text = openFileDialog1.FileName;
            }
        }

        #endregion

        #region Data Source Methods/Event Handlers

        /// <summary>
        /// Loads the test data sources into the <see cref="ListView"/>.
        /// </summary>
        private void LoadDataSources()
        {
            ListViewItem lvItem;

            lvDataSources.Items.Clear();

            List<DataSourceInfo> dataSources =
                _serviceFacade.GetDataSources(_currentTestRow["ConfigPath"].ToString());

            foreach (DataSourceInfo dataSource in dataSources)
            {
                lvItem = new ListViewItem(new string[]
                                              {
                                                  dataSource.Name, 
                                                  dataSource.DataSourcePath, 
                                                  dataSource.DataSourceBaseUri,
                                                  dataSource.DataTableName, 
                                                  dataSource.DataAccessMethod
                                              });

                lvDataSources.Items.Add(lvItem);
            }
        }

        #endregion
    }
}
