﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace RemoteTestRunner.UnitTesting
{
    /// <summary>
    /// Test class level object for tracking individual asserts in test methods that have multiple asserts.
    /// </summary>
    public class MultiAssertTestClassHelper : IDisposable
    {
        // MSTest tests can run in parallel.  This class must be able to support that functionality.
        private const string MultiAssertFileSuffix = ".MultiAssertOutput.xml";

        /// <summary>
        /// Initializes a new instance of the <see cref="MultiAssertTestClassHelper"/> class.
        /// </summary>
        /// <param name="testClassName">Name of the test class.</param>
        /// <param name="testDirectory">The test directory.</param>
        public MultiAssertTestClassHelper(string testClassName, string testDirectory)
        {
            TestClass = testClassName;
            OutputFile = testDirectory + "." + testClassName + MultiAssertFileSuffix;
            Functions = new List<TestMethodHelper>();
        }

        public string OutputFile { get; private set; }

        internal string TestClass { get; private set; }

        internal IList<TestMethodHelper> Functions { get; private set; }

        public TestMethodHelper GetFunctionHelper(TestContext testContext)
        {
            // create a new data item to hold the results.
            TestMethodHelper resultFunction = new TestMethodHelper(testContext);
            Functions.Add(resultFunction);
            return resultFunction;
        }

        #region IDisposable Members

        public void Dispose()
        {
            MultiAssertTestXmlRepository.Persist(this, OutputFile);
        }

        #endregion

        /// <summary>
        /// Function level routine for tracking individual asserts in test methods that have multiple asserts.
        /// </summary>
        public class TestMethodHelper : IDisposable
        {
            private TestContext _testContext;

            private int _failureCount;

            private Dictionary<string, object> _uniqueAsserts;

            internal TestMethodHelper(TestContext testContext)
            {
                _uniqueAsserts = new Dictionary<string, object>();
                Asserts = new List<AssertResult>();
                Inputs = new List<KeyValuePair<string, string>>();

                _testContext = testContext;
                _failureCount = 0;
                TestMethodName = testContext.TestName;
            }

            public IList<KeyValuePair<string, string>> Inputs { get; private set; }

            internal string TestMethodName { get; private set; }

            internal IList<AssertResult> Asserts { get; private set; }

            public TestMethodHelper AddInput(string key, object value)
            {
                Inputs.Add(new KeyValuePair<string, string>(key, value.ToString()));
                return this;
            }

            /// <summary>
            /// Ares the equal.  GhostDoc LOL!!  This function is a replacement to the standard AreEqual function.  
            /// It stores captures the results for the final report.
            /// </summary>
            /// <typeparam name="T">The type of the values to be compared.</typeparam>
            /// <param name="expected">The expected.</param>
            /// <param name="actual">The actual.</param>
            /// <param name="message">The message.</param>
            /// <param name="assertName">Name of the assert.</param>
            /// <returns>Returns an instance of the object for fluent calls.</returns>
            public TestMethodHelper AreEqual<T>(object expected, T actual, string message, string assertName)
            {
                AssertResult assertResult = new AssertResult(expected.ToString(), actual.ToString(), message, assertName);
                try
                {
                    assertResult = AreEqualInternal<T>((T)expected, actual, assertResult);
                }
                catch (InvalidCastException ex)
                {
                    // An exception can occur when casting to T.
                    var newException =
                        new InvalidCastException(
                            "Cannot cast 'expected' type of " + expected.GetType().Name + " to 'actual' type of " + typeof(T).Name + ".",
                            ex);

                    assertResult.TestPassed = false;
                    assertResult.ResultException = newException;
                    _failureCount++;
                    _testContext.WriteLine("Assert failed: " + newException.Message);
                }
                catch (Exception ex)
                {
                    // An exception can occur when casting to T.
                    assertResult.TestPassed = false;
                    assertResult.ResultException = ex;
                    _failureCount++;
                    _testContext.WriteLine("Assert failed: " + ex.Message);
                }

                return this;
            }

            /// <summary>
            /// This function is a replacement to the standard AreEqual function.  
            /// It stores captures the results for the final report.
            /// </summary>
            /// <typeparam name="T">The type of the values to be compared.</typeparam>
            /// <param name="expected">The expected.</param>
            /// <param name="actual">The actual.</param>
            /// <param name="message">The message.</param>
            /// <param name="assertName">Name of the assert.</param>
            /// <returns>Returns an instance of the object for fluent calls.</returns>
            public TestMethodHelper AreEqual<T>(T expected, T actual, string message, string assertName)
            {
                AssertResult assertResult = new AssertResult(expected.ToString(), actual.ToString(), message, assertName);
                AreEqualInternal<T>(expected, actual, assertResult);
                return this;
            }

            #region IDisposable Members

            /// <summary>
            /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
            /// </summary>
            public void Dispose()
            {
                if (_failureCount > 0)
                {
                    throw new AssertFailedException(_failureCount + " assert(s) failed.");
                }
            }

            #endregion

            /// <summary>
            /// Internal implementation of AreEqual that runs the test through the Microsoft assert.
            /// </summary>
            /// <typeparam name="T">The type of the values to be compared.</typeparam>
            /// <param name="expected">The expected.</param>
            /// <param name="actual">The actual.</param>
            /// <param name="assertResult">The assert result.</param>
            /// <returns>Returns the assert result.</returns>
            private AssertResult AreEqualInternal<T>(T expected, T actual, AssertResult assertResult)
            {
                try
                {
                    Asserts.Add(assertResult);

                    if (_uniqueAsserts.ContainsKey(assertResult.AssertName))
                    {
                        throw new ArgumentException("Assert names must be unique within a test method.", "assertResult");
                    }

                    _uniqueAsserts.Add(assertResult.AssertName, null);

                    // Call the Microsoft Assert.AreEqual
                    Assert.AreEqual<T>(expected, actual, assertResult.Message);
                    assertResult.TestPassed = true;
                }
                catch (AssertFailedException ex)
                {
                    assertResult.TestPassed = false;
                    _failureCount++;
                    _testContext.WriteLine("Assert failed: " + ex.Message);
                }
                catch (Exception ex)
                {
                    assertResult.TestPassed = false;
                    assertResult.ResultException = ex;
                    _failureCount++;
                    _testContext.WriteLine("Assert failed: " + ex.Message);
                }

                return assertResult;
            }

            /// <summary>
            /// Container object for storing the details for each executed assert.
            /// </summary>
            internal class AssertResult
            {
                public AssertResult(string expected, string actual, string message, string assertName)
                {
                    Expected = expected;
                    Actual = actual;
                    AssertName = assertName;
                    Message = message;
                }

                public string Expected { get; private set; }

                public string Actual { get; private set; }

                public string AssertName { get; private set; }

                public string Message { get; private set; }

                public bool TestPassed { get; set; }

                public Exception ResultException { get; set; }
            }
        }
    }
}